package com.bweb.controller;

import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.DateUtil;
import com.hai.common.utils.MemberValidateUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.model.HighAgentModel;
import com.hai.model.ResponseData;
import com.hai.model.UserInfoModel;
import com.hai.service.HighAgentService;
import com.hai.service.SecUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.ResponseBody;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;

/**
 * @Auther: 袁野
 * @Description:
 * @Date: 2021/3/25 17:19
 */
@Controller
@RequestMapping(value = "/highAgent")
@Api(value = "代理商接口")

public class HighAgentController {

    private static Logger log = LoggerFactory.getLogger(HighMerchantStoreController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private HighAgentService highAgentService;

    @Resource
    private SecUserService secUserService;

    @RequestMapping(value = "/insertAgent", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "新增代理商")
    public ResponseData insertAgent(@RequestBody HighAgentModel highAgentModel, HttpServletRequest request) {
        try {
            //发布人员
            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();
            if (userInfoModel.getMerchant() == null) {
                log.error("HighMerchantController -> insertAgent() error!","该主角色没有权限");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MENU_TREE_HAS_NOT_ERROR, "");
            }

            if (StringUtils.isBlank(highAgentModel.getAgentName())
                    || StringUtils.isBlank(highAgentModel.getAgentAddress())
                    || StringUtils.isBlank(highAgentModel.getAgentUser())
                    || StringUtils.isBlank(highAgentModel.getAgentPhone())
                    || StringUtils.isBlank(highAgentModel.getSecUser().getPassword())
                    || StringUtils.isBlank(highAgentModel.getSecUser().getTelephone())
            ) {
                log.error("HighMerchantController -> insertAgent() error!","参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            // 校验账号手机号是否存在
            if (secUserService.findByPhone(highAgentModel.getSecUser().getTelephone()) != null) {
                log.error("HighMerchantStoreController --> insertAgent() error!", "联系方式已被使用");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.TEL_EXISTED, "");
            }

            if (highAgentService.findByAgentName(highAgentModel.getAgentName())) {
                log.error("HighMerchantStoreController --> insertAgent() error!", "代理商名称已存在");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.AGENT_NAME, "");
            }

            // 校验账户手机号
            if (!MemberValidateUtil.validatePhone(highAgentModel.getSecUser().getTelephone())) {
                log.error("BsStudentController --> insertMerchant() error!", "请输入正确的电话号码");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_IS_ERROR, "");
            }

            // 校验账号手机号是否存在
            if (secUserService.findByPhone(highAgentModel.getSecUser().getTelephone()) != null) {
                log.error("BsStudentController --> insertMerchant() error!", "电话号码已被使用");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_EXISTED, "");
            }

            highAgentModel.setStatus(1); // 状态：0：不可用，1：可用
            highAgentModel.setCreateTime(new Date());
            highAgentModel.setOperatorId(userInfoModel.getSecUser().getId());
            highAgentModel.setOperatorName(userInfoModel.getSecUser().getUserName());
            highAgentService.insertAgentMsg(highAgentModel);

            return ResponseMsgUtil.success("新增成功");

        } catch (Exception e) {
            log.error("HighAgentController --> insertAgent() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value = "/updateAgent", method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "修改代理商")
    public ResponseData updateAgentMsg(@RequestBody HighAgentModel highAgentModel, HttpServletRequest request) {
        try {

            //发布人员
            SessionObject sessionObject = userCenter.getSessionObject(request);
            UserInfoModel userInfoModel = (UserInfoModel) sessionObject.getObject();
            if (userInfoModel.getMerchant() == null) {
                log.error("HighMerchantController -> insertAgent() error!","该主角色没有权限");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.MENU_TREE_HAS_NOT_ERROR, "");
            }

            if (highAgentModel.getId() == null
                    || StringUtils.isBlank(highAgentModel.getAgentName())
                    || StringUtils.isBlank(highAgentModel.getAgentAddress())
                    || StringUtils.isBlank(highAgentModel.getAgentUser())
                    || StringUtils.isBlank(highAgentModel.getAgentPhone())
                    || StringUtils.isBlank(highAgentModel.getSecUser().getPassword())
                    || StringUtils.isBlank(highAgentModel.getSecUser().getTelephone())
            ) {
                log.error("HighMerchantController -> insertAgent() error!","参数错误");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }



            // 校验账号手机号是否存在
            if (secUserService.findByPhone(highAgentModel.getSecUser().getTelephone()) != null) {
                log.error("HighMerchantStoreController --> insertAgent() error!", "联系方式已被使用");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.TEL_EXISTED, "");
            }

            if (highAgentService.findByAgentName(highAgentModel.getAgentName())) {
                log.error("HighMerchantStoreController --> insertAgent() error!", "代理商名称已存在");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.AGENT_NAME, "");
            }

            // 校验账户手机号
            if (!MemberValidateUtil.validatePhone(highAgentModel.getSecUser().getTelephone())) {
                log.error("BsStudentController --> insertMerchant() error!", "请输入正确的电话号码");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_IS_ERROR, "");
            }

            // 校验账号手机号是否存在
            if (secUserService.findByPhone(highAgentModel.getSecUser().getTelephone()) != null) {
                log.error("BsStudentController --> insertMerchant() error!", "电话号码已被使用");
                throw ErrorHelp.genException(SysCode.System, ErrorCode.PHONE_NUM_EXISTED, "");
            }

            highAgentModel.setStatus(1); // 状态：0：不可用，1：可用
            highAgentModel.setCreateTime(new Date());
            highAgentModel.setOperatorId(userInfoModel.getSecUser().getId());
            highAgentModel.setOperatorName(userInfoModel.getSecUser().getUserName());
            highAgentService.insertAgentMsg(highAgentModel);


            return ResponseMsgUtil.success("修改成功");

        } catch (Exception e) {
            log.error("BsPositionController --> addPosition() error!", e);
            return ResponseMsgUtil.exception(e);
        }
    }


}
