package com.cweb.controller;

import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hai.common.exception.ErrorCode;
import com.hai.common.exception.ErrorHelp;
import com.hai.common.exception.SysCode;
import com.hai.common.security.SessionObject;
import com.hai.common.security.UserCenter;
import com.hai.common.utils.MemberValidateUtil;
import com.hai.common.utils.ResponseMsgUtil;
import com.hai.entity.HighCoupon;
import com.hai.entity.HighMerchant;
import com.hai.entity.HighMerchantStore;
import com.hai.model.*;
import com.hai.service.HighCouponService;
import com.hai.service.HighMerchantService;
import com.hai.service.HighMerchantStoreService;
import com.hai.service.SecUserService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Auther: 胡锐
 * @Description:
 * @Date: 2021/3/9 20:58
 */
@Controller
@RequestMapping(value = "/highMerchantStore")
@Api(value = "商户门店接口")
public class HighMerchantStoreController {

    private static Logger log = LoggerFactory.getLogger(HighMerchantStoreController.class);

    @Autowired
    private UserCenter userCenter;

    @Resource
    private HighCouponService highCouponService;

    @Resource
    private HighMerchantStoreService highMerchantStoreService;

    @Resource
    private HighMerchantService highMerchantService;

    @Resource
    private SecUserService secUserService;

    @RequestMapping(value="/getMerchantStoreById",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据id查询商户门店")
    public ResponseData getMerchantStoreById(@RequestParam(name = "id", required = true) Long id) {
        try {

            return ResponseMsgUtil.success(highMerchantStoreService.getMerchantStoreById(id));

        } catch (Exception e) {
            log.error("HighMerchantStoreController -> getMerchantStoreById() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/getStoreListByCoupon",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "根据卡卷查询门店列表")
    public ResponseData getStoreListByCoupon(@RequestParam(name = "couponId", required = true) Long couponId,
                                             @RequestParam(name = "longitude", required = true) String longitude,
                                             @RequestParam(name = "latitude", required = true) String latitude) {
        try {

            HighCoupon coupon = highCouponService.getCouponById(couponId);
            if (coupon == null) {
                return ResponseMsgUtil.success(new PageInfo<>());
            }
            Map<String,Object> map = new HashMap<>();
            map.put("merchantId", coupon.getMerchantId());
            map.put("longitude", longitude);
            map.put("latitude", latitude);

            List<HighMerchantStore> storeList = highMerchantStoreService.getStoreListByLongitude(map);
            if (storeList != null && storeList.size() > 0) {
                HighMerchant merchant = highMerchantService.getMerchantById(coupon.getMerchantId());
                for (HighMerchantStore store : storeList) {
                    store.setExt1(merchant.getMerchantLogo());
                }
            }
            return ResponseMsgUtil.success(storeList);

        } catch (Exception e) {
            log.error("HighMerchantStoreController -> getStoreListByCoupon() error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }


}
